@@ -76,6 +76,8 @@ class ClassMapEntry:
         has_instances (bool, optional): Whether each label of this class is a separate instance. Defaults to True.
         ignore_in_eval (bool, optional): Whether to ignore this class while evaluating metrics. Defaults to False.
         train_color (Optional[Union[Tuple[int, int, int], List[int]]], optional): A length 3 list/tuple of RGB int values for showing inferences of this class. Defaults to None.
+        keypoint_names (Optional[List[str]], optional): List of keypoint names that exist for this class. Indicies of keypoints in this list are used for the skeletons.
+        skeleton (Optional[List[Union[Tuple[int, int], List[int]]]], optional): List of integer pairs that reference indicies in keypoint_names that represent the skeleton for this class.
     """
 
     def __init__(
@@ -91,6 +93,8 @@ class ClassMapEntry:
         has_instances: bool = True,
         ignore_in_eval: bool = False,
         train_color: Optional[UserProvidedColor] = None,
+        keypoint_names: Optional[List[str]] = None,
+        skeleton: Optional[List[Union[Tuple[int, int], List[int]]]] = None,
     ) -> None:
 
         # Set defaults for more complex class mapping fields if not set
@@ -152,6 +156,31 @@ class ClassMapEntry:
                     "Argument 'train_color' must have integer entries in the range [0,255]."
                 )
 
+        # Keypoints skeleton
+        if keypoint_names is not None:
+            if not isinstance(keypoint_names, list):
+                raise Exception("Argument 'keypoint_names' must be a list of strings.")
+            for keypoint_name in keypoint_names:
+                if not isinstance(keypoint_name, str):
+                    raise Exception(
+                        "Argument 'keypoint_names' must be a list of strings."
+                    )
+        if skeleton is not None:
+            if keypoint_names is None:
+                raise Exception(
+                    "Argument 'keypoint_names' cannot be None when using Argument 'skeleton'."
+                )
+            if not isinstance(skeleton, list):
+                raise Exception("Argument 'skeleton' must be a list of integer pairs.")
+            for bone in skeleton:
+                if type(bone) not in [list, tuple] or len(bone) != 2:
+                    raise Exception(
+                        "Argument 'skeleton' must be a list of integer pairs."
+                    )
+                for i in bone:
+                    if i >= len(keypoint_names):
+                        raise Exception("Index out of range in argument 'skeleton'.")
+
         self.name = name
         self.class_id = class_id
         self.train_name = train_name
@@ -162,6 +191,8 @@ class ClassMapEntry:
         self.ignore_in_eval = ignore_in_eval
         self.color = list(color)
         self.train_color = list(train_color)
+        self.keypoint_names = keypoint_names
+        self.skeleton = skeleton
 
     def to_dict(self) -> Dict[str, Any]:
         """Returns a dictified form of this data structure.
@@ -180,6 +211,8 @@ class ClassMapEntry:
             "ignore_in_eval": self.ignore_in_eval,
             "color": self.color,
             "train_color": self.train_color,
+            "keypoint_names": self.keypoint_names,
+            "skeleton": self.skeleton,
         }
 
 
@@ -344,6 +377,8 @@ class ClassMapUpdateEntry:
         class_id (Optional[int], optional): The int representation of the class
         color (Optional[Union[Tuple[int, int, int], List[int]]], optional): A length 3 list/tuple of RGB int values
         train_color (Optional[Union[Tuple[int, int, int], List[int]]], optional): A length 3 list/tuple of RGB int values for showing inferences of this class. Defaults to None.
+        keypoint_names (Optional[List[str]], optional): List of keypoint names that exist for this class. Indicies of keypoints in this list are used for the skeletons.
+        skeleton (Optional[List[Union[Tuple[int, int], List[int]]]], optional): List of integer pairs that reference indicies in keypoint_names that represent the skeleton for this class.
     """
 
     def __init__(
@@ -353,6 +388,8 @@ class ClassMapUpdateEntry:
         class_id: Optional[int] = None,
         color: Optional[UserProvidedColor] = None,
         train_color: Optional[UserProvidedColor] = None,
+        keypoint_names: Optional[List[str]] = None,
+        skeleton: Optional[List[Union[Tuple[int, int], List[int]]]] = None,
     ) -> None:
         # Assert on types
         # Name
@@ -408,10 +445,37 @@ class ClassMapUpdateEntry:
                 "ClassMapUpdateEntry must include an update of either 'color' or 'train_color'"
             )
 
+        # Keypoints skeleton
+        if keypoint_names is not None:
+            if not isinstance(keypoint_names, list):
+                raise Exception("Argument 'keypoint_names' must be a list of strings.")
+            for keypoint_name in keypoint_names:
+                if not isinstance(keypoint_name, str):
+                    raise Exception(
+                        "Argument 'keypoint_names' must be a list of strings."
+                    )
+        if skeleton is not None:
+            if keypoint_names is None:
+                raise Exception(
+                    "Argument 'keypoint_names' cannot be None when using Argument 'skeleton'."
+                )
+            if not isinstance(skeleton, list):
+                raise Exception("Argument 'skeleton' must be a list of integer pairs.")
+            for bone in skeleton:
+                if type(bone) not in [list, tuple] or len(bone) != 2:
+                    raise Exception(
+                        "Argument 'skeleton' must be a list of integer pairs."
+                    )
+                for i in bone:
+                    if i >= len(keypoint_names):
+                        raise Exception("Index out of range in argument 'skeleton'.")
+
         self.name = name
         self.class_id = class_id
         self.color = color
         self.train_color = train_color
+        self.keypoint_names = keypoint_names
+        self.skeleton = skeleton
 
     def to_dict(self) -> Dict[str, Any]:
         """Returns a dictified form of this data structure.
@@ -425,4 +489,6 @@ class ClassMapUpdateEntry:
             "id": self.class_id,
             "color": self.color,
             "train_color": self.train_color,
+            "keypoint_names": self.keypoint_names,
+            "skeleton": self.skeleton,
         }
