@@ -12,6 +12,7 @@ import uuid
 import math
 import json
 import pandas as pd
+from requests.packages.urllib3.util import retry
 import pyarrow as pa
 from uuid import uuid4
 from io import IOBase
@@ -56,6 +57,8 @@ from .sampling_agent import RandomSamplingAgent, SamplingScoreDict, SamplingAgen
 from .embedding_distance_sampling import EmbeddingDistanceSamplingAgent
 
 CollectionAssetType = Literal["sample_frame", "embedding"]
+SortStatus = Literal["UNSORTED", "DISCARDED", "ACCEPTED"]
+SimilarityResultProcessingStatus = Literal["ENQUEUED", "RUNNING", "DONE"]
 
 DEFAULT_STATISTICS = {
     "n": 0,
@@ -217,6 +220,104 @@ class CollectionClient(Client):
                 continue
         return True
 
+    def get_unlabeled_results_for_issue(
+        self,
+        *,
+        issue_uuid: str,
+        unlabeled_dataset_name: Optional[str],
+        sort_status: Optional[SortStatus] = "UNSORTED",
+        unlabeled_only: Optional[bool] = True,
+        retrigger_search: Optional[bool] = False,
+    ) -> List[Dict[str, Any]]:
+        """returns similar results from unlabeled dataset search
+
+        Args:
+            issue_uuid (str): the uuid of the seed issue
+            unlabeled_dataset_name (str): the name of the unlabeled search dataset
+            sort_status (Optional[SortStatus]): filter the returned collection frames by status (default UNSORTED)
+            unlabeled_only (Optional[bool]): return only collection frames that haven't been sent to labeling
+            retrigger_search (Optional[bool]): retrigger similarity search (NOTE: this only works for searches against unlabeled datasets. It will poll + wait for the result)
+
+        Returns:
+            List[Dict[str, Any]]: List of CollectionFrames for a given issue
+        """
+
+        if not unlabeled_dataset_name and retrigger_search:
+            raise Exception(
+                "Can't retrigger search for Python client collection campaigns, only campaigns against unlabeled datasets"
+            )
+
+        collection_campaigns = self._get_all_campaigns(target_issue_uuids=[issue_uuid])
+        if not collection_campaigns:
+            raise Exception(
+                f"Issue {issue_uuid} does not support similarity search because it does not have an associated collection campaign."
+            )
+
+        campaign_id = collection_campaigns[0]["id"]
+        project_name = collection_campaigns[0]["project_name"]
+
+        if retrigger_search:
+            print(f"Retriggering unlabeled similarity search for issue {issue_uuid}...")
+
+            search_payload = {
+                "search_dataset_name": unlabeled_dataset_name,
+                "collection_campaign_id": campaign_id,
+            }
+
+            r = requests_retry.post(
+                self.api_endpoint
+                + f"/projects/{project_name}/issues/{issue_uuid}/find_similar_elements",
+                headers=self._get_creds_headers(),
+                json=search_payload,
+            )
+            raise_resp_exception_error(r)
+
+            # TODO: possibly insert sleep in case status isn't set right away?
+
+            retry_count = 0
+            processing_status: SimilarityResultProcessingStatus = "ENQUEUED"
+            while processing_status != "DONE":
+                if retry_count != 0:
+                    sleep_time = 2 ** retry_count
+                    print(f"Still running. Sleeping for {sleep_time} seconds...")
+                    time.sleep(sleep_time)
+
+                print("Polling for search status...")
+                polling_params = {"search_dataset_name": unlabeled_dataset_name}
+                r = requests_retry.get(
+                    self.api_endpoint
+                    + f"/projects/{project_name}/issues/{issue_uuid}/similar_element_info",
+                    headers=self._get_creds_headers(),
+                    params=polling_params,
+                )
+
+                raise_resp_exception_error(r)
+                poll_result: Dict[str, Any] = r.json()
+                processing_status = poll_result["similar_elts_processing_status"]
+                retry_count += 1
+
+            print("Similarity search complete, fetching results...")
+
+        fetch_params: Dict[str, Any] = {
+            "unlabeled_only": unlabeled_only,
+            "sort_status": sort_status,
+            "python_client": True,
+        }
+
+        if unlabeled_dataset_name:
+            fetch_params["search_dataset"] = unlabeled_dataset_name
+
+        r = requests_retry.get(
+            self.api_endpoint
+            + f"/projects/{project_name}/collection_frames/for_issue/{issue_uuid}",
+            headers=self._get_creds_headers(),
+            params=fetch_params,
+        )
+
+        raise_resp_exception_error(r)
+        result: List[Dict[str, Any]] = r.json()
+        return result
+
     def _read_rows_from_disk(self, file_path: str) -> List[Dict[str, Any]]:
         """reads temp files from disk and loads the dicts in them into memory
 
