@@ -16,6 +16,7 @@ from typing_extensions import Protocol, TypedDict
 
 from .util import (
     ResizeMode,
+    UpdateType,
     _is_one_gb_available,
     assert_valid_name,
     add_object_user_attrs,
@@ -28,6 +29,7 @@ from .util import (
     ORIENTATION_KEYS,
     KEYPOINT_KEYS,
     MAX_FRAMES_PER_BATCH,
+    DEFAULT_SENSOR_ID,
     GtLabelEntryDict,
     FrameEmbeddingDict,
     CropEmbeddingDict,
@@ -62,12 +64,14 @@ class LabeledFrame(BaseFrame):
         date_captured: Optional[str] = None,
         device_id: Optional[str] = None,
         reuse_latest_embedding: Optional[bool] = False,
+        update_type: UpdateType = "ADD",
     ) -> None:
         super(LabeledFrame, self).__init__(
             frame_id=frame_id,
             date_captured=date_captured,
             device_id=device_id,
             reuse_latest_embedding=reuse_latest_embedding,
+            update_type=update_type,
         )
 
         self._labels = GTLabelSet(
@@ -148,7 +152,7 @@ class LabeledFrame(BaseFrame):
     def add_label_text_token(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         index: int,
         token: str,
@@ -160,7 +164,7 @@ class LabeledFrame(BaseFrame):
         """Add a label for a text token.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             index (int): the index of this token in the text
             token (str): the text content of this token
@@ -232,7 +236,7 @@ class LabeledFrame(BaseFrame):
     def add_label_2d_bbox(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         top: Union[int, float],
@@ -247,7 +251,7 @@ class LabeledFrame(BaseFrame):
         """Add a label for a 2D bounding box.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             top (int or float): The top of the box in pixels
@@ -277,7 +281,7 @@ class LabeledFrame(BaseFrame):
     def add_label_2d_line_segment(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         x1: Union[int, float],
@@ -292,7 +296,7 @@ class LabeledFrame(BaseFrame):
         """Add a label for a 2D line segment.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             x1 (int or float): The x-coord of the first vertex in pixels
@@ -320,15 +324,19 @@ class LabeledFrame(BaseFrame):
     def add_label_2d_keypoints(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
-        top: Union[int, float],
-        left: Union[int, float],
-        width: Union[int, float],
-        height: Union[int, float],
         keypoints: List[Dict[KEYPOINT_KEYS, Union[int, float, str]]],
         confidence: Optional[float] = None,
+        top: Optional[Union[int, float]] = None,
+        left: Optional[Union[int, float]] = None,
+        width: Optional[Union[int, float]] = None,
+        height: Optional[Union[int, float]] = None,
+        polygons: Optional[
+            List[Dict[POLYGON_VERTICES_KEYS, List[Tuple[Union[int, float]]]]]
+        ] = None,
+        center: Optional[List[Union[int, float]]] = None,
         user_attrs: Optional[Dict[str, Any]] = None,
     ) -> None:
         """Add a label for a 2D keypoints task.
@@ -339,34 +347,38 @@ class LabeledFrame(BaseFrame):
             'name': string name of the keypoint
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
-            top (int or float): The top of the box in pixels
-            left (int or float): The left of the box in pixels
-            width (int or float): The width of the box in pixels
-            height (int or float): The height of the box in pixels
             keypoints (list of dicts): The keypoints of this detection
             confidence (float, optional): (Unlabeled Datasets Only) The confidence between 0.0 and 1.0 of the proposed label.
+            top (int or float, optional): The top of the bounding box in pixels. Defaults to None.
+            left (int or float, optional): The left of the bounding box in pixels. Defaults to None.
+            width (int or float, optional): The width of the bounding box in pixels. Defaults to None.
+            height (int or float, optional): The height of the bounding box in pixels. Defaults to None.
+            polygons (list of dicts, optional): The polygon geometry. Defaults to None.
+            center (list of ints or floats, optional): The center point of the polygon instance. Defaults to None.
             user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
         """
         self._labels.add_2d_keypoints(
             sensor_id=sensor_id,
             label_id=label_id,
             classification=classification,
+            keypoints=keypoints,
+            confidence=confidence,
             top=top,
             left=left,
             width=width,
             height=height,
-            keypoints=keypoints,
-            confidence=confidence,
+            polygons=polygons,
+            center=center,
             user_attrs=user_attrs,
         )
 
     def add_label_2d_polygon_list(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         polygons: List[Dict[POLYGON_VERTICES_KEYS, List[Tuple[Union[int, float]]]]],
@@ -393,7 +405,7 @@ class LabeledFrame(BaseFrame):
                     }
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             polygons (list of dicts): The polygon geometry
@@ -414,7 +426,7 @@ class LabeledFrame(BaseFrame):
     def add_label_2d_instance_seg(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         mask_url: Optional[str] = None,
         mask_data: Optional[Any] = None,
@@ -424,7 +436,7 @@ class LabeledFrame(BaseFrame):
         """Add a label for 2D instance segmentation.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             mask_url (str, optional): URL to the pixel mask png.
             mask_data (ndarray, optional): ndarray of pixel mask data, shape [height, width]
@@ -455,7 +467,7 @@ class LabeledFrame(BaseFrame):
     def add_label_2d_semseg(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         mask_url: Optional[str] = None,
         mask_data: Optional[Any] = None,
@@ -464,7 +476,7 @@ class LabeledFrame(BaseFrame):
         """Add a label for 2D semseg.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             mask_url (str, optional): URL to the pixel mask png.
             mask_data (ndarray, optional): ndarray of pixel mask data, shape [height, width]
@@ -484,7 +496,7 @@ class LabeledFrame(BaseFrame):
     def add_label_2d_classification(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         confidence: Optional[float] = None,
@@ -494,7 +506,7 @@ class LabeledFrame(BaseFrame):
         """Add a label for 2D classification.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             confidence (float, optional): (Unlabeled Datasets Only) The confidence between 0.0 and 1.0 of the proposed label.
@@ -576,6 +588,10 @@ class LabeledDataset:
     backfilled_crops_flag: bool
     backfilled_frame_embs_flag: bool
     labels_with_confidence: bool
+    crop_embedding_dim: Optional[int]
+    frame_embedding_dim: Optional[int]
+    sample_frame_embeddings: List[List[float]]
+    sample_crop_embeddings: List[List[float]]
 
     def __init__(self, *, pipeline_mode: str = "BATCH") -> None:
         self._frames = []
@@ -612,6 +628,10 @@ class LabeledDataset:
         self.pipeline_mode = pipeline_mode
         self.backfilled_crops_flag = False
         self.backfilled_frame_embs_flag = False
+        self.crop_embedding_dim = None
+        self.frame_embedding_dim = None
+        self.sample_frame_embeddings = []
+        self.sample_crop_embeddings = []
         self.labels_with_confidence = False
 
     def _cleanup_temp_dir(self) -> None:
@@ -874,6 +894,66 @@ class LabeledDataset:
         self._label_class_ids_set.update(frame._labels._seen_label_class_ids)
         self._label_classes_set.update(frame._all_label_classes())
         self._frame_summaries.append(frame._to_summary())
+
+        if self.frame_embedding_dim is None:
+            self.frame_embedding_dim = frame.frame_embedding_dim
+        elif self.frame_embedding_dim != frame.frame_embedding_dim:
+            raise Exception(
+                f"Length of frame embeddings must be the same, existing embeddings are of dimension {self.frame_embedding_dim} but new embedding has dimension {frame.frame_embedding_dim}"
+            )
+
+        frame_embedding = (frame.embedding or {"embedding": None}).get("embedding")
+        if frame_embedding and len(self.sample_frame_embeddings) < 5:
+            self.sample_frame_embeddings.append(frame_embedding)
+
+        if len(frame._labels._label_ids_set) > 0:
+            if len(frame._labels.crop_embeddings) > 0 and len(
+                frame._labels.crop_embeddings
+            ) != len(frame._labels._label_ids_set):
+                raise Exception(
+                    f"Embeddings must be provided for all crops or no crops. Embeddings are provided for partial crops for frame {frame.frame_id}."
+                )
+
+            crop_embedding_dim = (
+                frame._labels.crop_embedding_dim
+                if frame._labels.crop_embedding_dim
+                else -1
+            )
+            if self.crop_embedding_dim is None:
+                self.crop_embedding_dim = crop_embedding_dim
+            elif self.crop_embedding_dim != crop_embedding_dim:
+                raise Exception(
+                    f"Length of crop embeddings must be the same, existing embeddings are of dimension {self.crop_embedding_dim} but new embedding has dimension {crop_embedding_dim}"
+                )
+
+            if (
+                len(frame._labels.crop_embeddings) > 0
+                and len(self.sample_crop_embeddings) < 5
+            ):
+                remaining = 5 - len(self.sample_crop_embeddings)
+                self.sample_crop_embeddings += [
+                    x["embedding"] for x in frame._labels.crop_embeddings[0:remaining]
+                ]
+
+        if len(self._frames) >= MAX_FRAMES_PER_BATCH:
+            self._flush_to_disk()
+
+    def delete_frame(self, frame_id: str) -> None:
+        """Delete a frame to this dataset.
+
+        Args:
+            frame_id: A frame_id in the dataset.
+        """
+        if frame_id in self._frame_ids_set:
+            raise Exception("Attempted to add and delete the same frame id.")
+
+        self._frames.append(
+            LabeledFrame(
+                frame_id=frame_id, reuse_latest_embedding=True, update_type="DELETE"
+            )
+        )
+        self._frame_ids_set.add(frame_id)
+
         if len(self._frames) >= MAX_FRAMES_PER_BATCH:
             self._flush_to_disk()
 
@@ -884,6 +964,8 @@ class LabeledDataset:
             filelike (filelike): The destination file-like to write to.
         """
         for frame in self._frames:
+            if frame.update_type != "ADD":
+                continue
             row = frame.to_dict()
             row["label_data"] = frame._labels.to_dict()["label_data"]
             filelike.write(json.dumps(row) + "\n")
@@ -938,14 +1020,8 @@ class LabeledDataset:
         # to avoid error-prone inline manual casts like this?
         frames_with_embs = cast(List[GTFrameWithEmbeddings], self._frames)
 
-        # Get the first frame embedding dimension
-        frame_embedding_dim = len(frames_with_embs[0].embedding["embedding"])
-        # Get the first crop embedding dimension
-        crop_embedding_dim = 1
         for frame in frames_with_embs:
             if frame._labels.crop_embeddings:
-                first_crop_emb = frame._labels.crop_embeddings[0]
-                crop_embedding_dim = len(first_crop_emb["embedding"])
                 break
 
         frame_ids = np.empty((count), dtype=object)
@@ -981,7 +1057,6 @@ class LabeledDataset:
         """
         for frame in self._frames:
             row = frame.to_dict()
-            row["change"] = "ADD"
             filelike.write(json.dumps(row) + "\n")
 
     def write_labels_to_file_streaming(self, filelike: IO[Any]) -> None:
@@ -992,7 +1067,6 @@ class LabeledDataset:
         """
         for frame in self._frames:
             row = frame._labels.to_dict()
-            row["change"] = "ADD"
             filelike.write(json.dumps(row) + "\n")
 
     def write_frame_embeddings_to_file_streaming(self, filelike: IO[Any]) -> None:
