@@ -20,6 +20,7 @@ from .util import (
     ALL_ORIENTATION_KEYS,
     ALL_POSITION_KEYS,
     CAMERA_MODELS,
+    DEFAULT_SENSOR_ID,
     ResizeMode,
     _is_one_gb_available,
     assert_and_get_valid_iso_date,
@@ -29,6 +30,7 @@ from .util import (
     is_valid_float,
     is_valid_number,
     mark_temp_directory_complete,
+    extract_illegal_characters,
     TYPE_PRIMITIVE_TO_STRING_MAP,
     USER_METADATA_SEQUENCE,
     USER_METADATA_PRIMITIVE_TYPES,
@@ -84,6 +86,8 @@ class BaseFrame:
     embedding: Optional[FrameEmbeddingDict]
     _label_ids_set: Set[str]
     reuse_latest_embedding: Optional[bool]
+    frame_embedding_dim: int
+    update_type: UpdateType
 
     def __init__(
         self,
@@ -92,12 +96,16 @@ class BaseFrame:
         date_captured: Optional[str] = None,
         device_id: Optional[str] = None,
         reuse_latest_embedding: Optional[bool] = False,
+        update_type: UpdateType = "ADD",
     ) -> None:
         if not isinstance(frame_id, str):
             raise Exception("frame ids must be strings")
 
-        if "/" in frame_id:
-            raise Exception("frame ids cannot contain slashes (/)")
+        illegal_frame_id_characters = extract_illegal_characters(frame_id)
+        if illegal_frame_id_characters:
+            raise Exception(
+                f"frame ids cannot contain the following character(s): {illegal_frame_id_characters}"
+            )
 
         self.frame_id = frame_id
 
@@ -117,9 +125,11 @@ class BaseFrame:
         self.user_metadata = []
         self.embedding = None
         self.reuse_latest_embedding = reuse_latest_embedding
+        self.frame_embedding_dim = -1
 
         self._coord_frame_ids_set = set()
         self._label_ids_set = set()
+        self.update_type = update_type
 
     def _add_coordinate_frame(self, coord_frame_obj: Dict[str, str]) -> None:
         """Add coordinate frame to the dataset frame
@@ -160,12 +170,14 @@ class BaseFrame:
                 )
 
         if not self.embedding:
+            self.frame_embedding_dim = len(embedding)
             self.embedding = {
                 "task_id": self.frame_id,
                 "model_id": model_id,
                 "date_generated": str(datetime.datetime.now()),
                 "embedding": embedding,
             }
+            self.reuse_latest_embedding = False
 
     def add_user_metadata(
         self,
@@ -351,7 +363,7 @@ class BaseFrame:
         x, y, z, intensity (optional), range (optional)
 
         Args:
-            sensor_id (str): sensor id
+            sensor_id (str): ID of sensor that generates this datapoint (optional if there is only 1 datapoint per frame)
             pcd_url (str): URL to PCD formated data
             coord_frame_id (Optional[str], optional): The coordinate frame id. Defaults to None.
             date_captured (Optional[str], optional): ISO formatted date. Defaults to None.
@@ -415,7 +427,7 @@ class BaseFrame:
         pointcloud_url are provided.
 
         Args:
-            sensor_id (str): Sensor id
+            sensor_id (str): ID of sensor that generates this datapoint (optional if there is only 1 datapoint per frame)
             pointcloud_url (Optional[str]): URL for the pointcloud: float32 [x1, y1, z1, x2, y2, z2, ...]
             kitti_format_url (Optional[str]): URL for the pointcloud + intensity: float32 [x1, y1, z1, i1, x2, y2, z2, i2, ...]
             intensity_url (Optional[str]): URL for the Intensity Pointcloud: unsigned int32 [i1, i2, ...]
@@ -492,7 +504,7 @@ class BaseFrame:
         """Add a .obj file to the frame for text based geometry
 
         Args:
-            sensor_id (str): sensor id
+            sensor_id (str): ID of sensor that generates this datapoint (optional if there is only 1 datapoint per frame)
             obj_url (str): URL to where the object is located
             coord_frame_id (Optional[str], optional): ID for the coordinate frame. Defaults to None.
             date_captured (Optional[str], optional): ISO formatted date. Defaults to None.
@@ -536,7 +548,7 @@ class BaseFrame:
     def add_image(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         image_url: str,
         preview_url: Optional[str] = None,
         date_captured: Optional[str] = None,
@@ -548,7 +560,7 @@ class BaseFrame:
         """Add an image "sensor" data to this frame.
 
         Args:
-            sensor_id (str): The id of this sensor.
+            sensor_id (str): ID of sensor that generates this datapoint (optional if there is only 1 datapoint per frame)
             image_url (str): The URL to load this image data.
             preview_url (Optional[str], optional): A URL to a compressed version of the image. It must be the same pixel dimensions as the original image. Defaults to None.
             date_captured (Optional[str], optional): ISO formatted date. Defaults to None.
@@ -621,7 +633,7 @@ class BaseFrame:
     def add_image_overlay(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         overlay_id: str,
         image_url: str,
         date_captured: Optional[str] = None,
@@ -633,7 +645,7 @@ class BaseFrame:
         """Add an image overlay for the given "sensor" to this frame.
 
         Args:
-            sensor_id (str): The id of this sensor.
+            sensor_id (str): ID of sensor that generates this datapoint (optional if there is only 1 datapoint per frame)
             overlay_id (str): The id of this overlay.
             image_url (str): The URL to load this image data.
             date_captured (Optional[str], optional): ISO formatted date. Defaults to None.
@@ -689,7 +701,7 @@ class BaseFrame:
     def add_audio(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         audio_url: str,
         date_captured: Optional[str] = None,
         mirror_asset: Optional[bool] = False,
@@ -697,7 +709,7 @@ class BaseFrame:
         """Add an audio "sensor" data to this frame.
 
         Args:
-            sensor_id (str): The id of this sensor.
+            sensor_id (str): ID of sensor that generates this datapoint (optional if there is only 1 datapoint per frame)
             audio_url (str): The URL to load this audio data (mp3, etc.).
             date_captured (str, optional): ISO formatted date. Defaults to None.
             mirror_asset (Optional[str]), optional): request this asset in this frame to be saved to Aquarium Storage. Useful for Short duration signed URL assets.
@@ -731,7 +743,7 @@ class BaseFrame:
     def add_video(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         video_url: str,
         date_captured: Optional[str] = None,
         mirror_asset: Optional[bool] = False,
@@ -739,7 +751,7 @@ class BaseFrame:
         """Add a video "sensor" data to this frame.
 
         Args:
-            sensor_id (str): The id of this sensor.
+            sensor_id (str): ID of sensor that generates this datapoint (optional if there is only 1 datapoint per frame)
             video_url (str): The URL to load this video data (mp4, webm, etc.).
             date_captured (str, optional): ISO formatted date. Defaults to None.
             mirror_asset (Optional[str]), optional): request this asset in this frame to be saved to Aquarium Storage. Useful for Short duration signed URL assets.
@@ -1030,12 +1042,16 @@ class BaseFrame:
         )
 
     def add_text(
-        self, *, sensor_id: str, text: str, date_captured: Optional[str] = None
+        self,
+        *,
+        sensor_id: str = DEFAULT_SENSOR_ID,
+        text: str,
+        date_captured: Optional[str] = None,
     ) -> None:
         """Add a text "sensor" data to this frame.
 
         Args:
-            sensor_id (str): The id of this sensor.
+            sensor_id (str): ID of sensor that generates this datapoint (optional if there is only 1 datapoint per frame)
             text (str): The text body.
             date_captured (str, optional): ISO formatted date. Defaults to None.
         """
@@ -1090,6 +1106,7 @@ class BaseFrame:
 
         row["user_metadata_types"] = user_metadata_types
         row["user_metadata_modes"] = user_metadata_modes
+        row["change"] = self.update_type
         return row
 
 
