@@ -17,7 +17,9 @@ from typing_extensions import Protocol
 
 from .util import (
     DUMMY_FRAME_EMBEDDING,
+    DEFAULT_SENSOR_ID,
     ResizeMode,
+    UpdateType,
     _is_one_gb_available,
     create_temp_directory,
     mark_temp_directory_complete,
@@ -41,14 +43,22 @@ class InferenceFrameWithEmbeddings(Protocol):
 
 class InferencesFrame(InferenceLabelSet):
     embedding: Optional[FrameEmbeddingDict]
+    frame_embedding_dim: int
+    update_type: UpdateType
 
     def __init__(
-        self, *, frame_id: str, reuse_latest_embedding: Optional[bool] = False
+        self,
+        *,
+        frame_id: str,
+        reuse_latest_embedding: Optional[bool] = False,
+        update_type: UpdateType = "ADD",
     ) -> None:
         super().__init__(
             frame_id=frame_id, reuse_latest_embedding=reuse_latest_embedding
         )
         self.embedding = None
+        self.frame_embedding_dim = -1
+        self.update_type = update_type
 
     def add_frame_embedding(
         self, *, embedding: List[float], model_id: str = ""
@@ -75,6 +85,44 @@ class InferencesFrame(InferenceLabelSet):
                 "date_generated": str(datetime.datetime.now()),
                 "embedding": embedding,
             }
+            self.frame_embedding_dim = len(embedding)
+
+    def to_dict(self) -> Dict[str, Any]:
+        row = super().to_dict()
+        row["change"] = self.update_type
+        return row
+
+    def validate_and_backfill_crop_embeddings(self) -> bool:
+        """Determines if there are frame embeddings but not Crop embeddings
+        Backfills crop embeddings if Frame embeddings exist
+        Returns True if backfill was necessary
+
+        Returns:
+            bool: returns if backfill on frame was necessary
+        """
+
+        # Handle case where Frame embedding is still None
+        if not self.embedding:
+            return False
+
+        if len(self.crop_embeddings) > 0:
+            return False
+
+        return_flag = False
+
+        for label in self.label_data:
+            if label.get("label_type", "") in [
+                "CLASSIFICATION_2D",
+                "CLASSIFICATION_3D",
+            ]:
+                return False
+            label_id = label["uuid"]
+
+            self.add_crop_embedding(
+                label_id=label_id, embedding=[0.0, 0.0, 0.0], model_id="crop_default"
+            )
+            return_flag = True
+        return return_flag
 
     def add_embedding(
         self,
@@ -88,7 +136,7 @@ class InferencesFrame(InferenceLabelSet):
     def add_inference_2d_bbox(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         top: Union[int, float],
@@ -103,7 +151,7 @@ class InferencesFrame(InferenceLabelSet):
         """Add an inference for a 2D bounding box.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             top (int or float): The top of the box in pixels
@@ -133,7 +181,7 @@ class InferencesFrame(InferenceLabelSet):
     def add_inference_2d_line_segment(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         x1: Union[int, float],
@@ -148,7 +196,7 @@ class InferencesFrame(InferenceLabelSet):
         """Add an inference for a 2D line segment.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             x1 (int or float): The x-coord of the first vertex in pixels
@@ -175,7 +223,7 @@ class InferencesFrame(InferenceLabelSet):
     def add_inference_text_token(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         index: int,
         token: str,
@@ -187,7 +235,7 @@ class InferencesFrame(InferenceLabelSet):
         """Add a label for a text token.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             index (int): the index of this token in the text
             token (str): the text content of this token
@@ -251,18 +299,22 @@ class InferencesFrame(InferenceLabelSet):
     def add_inference_2d_keypoints(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
-        top: Union[int, float],
-        left: Union[int, float],
-        width: Union[int, float],
-        height: Union[int, float],
         keypoints: List[Dict[KEYPOINT_KEYS, Union[int, float, str]]],
         confidence: float,
+        top: Optional[Union[int, float]] = None,
+        left: Optional[Union[int, float]] = None,
+        width: Optional[Union[int, float]] = None,
+        height: Optional[Union[int, float]] = None,
+        polygons: Optional[
+            List[Dict[POLYGON_VERTICES_KEYS, List[Tuple[Union[int, float]]]]]
+        ] = None,
+        center: Optional[List[Union[int, float]]] = None,
         user_attrs: Optional[Dict[str, Any]] = None,
     ) -> None:
-        """Add an inference for a 2D keypoints task.
+        """Add an inference for a 2D keypoints task. Can include a bounding box and/or polygon.
 
         A keypoint is a dictionary of the form:
             'x': x-coordinate in pixels
@@ -270,34 +322,38 @@ class InferencesFrame(InferenceLabelSet):
             'name': string name of the keypoint
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
-            top (int or float): The top of the box in pixels
-            left (int or float): The left of the box in pixels
-            width (int or float): The width of the box in pixels
-            height (int or float): The height of the box in pixels
             keypoints (list of dicts): The keypoints of this detection
             confidence (float): The confidence between 0.0 and 1.0 of the prediction
+            top (int or float, optional): The top of the bounding box in pixels. Defaults to None.
+            left (int or float, optional): The left of the bounding box in pixels. Defaults to None.
+            width (int or float, optional): The width of the bounding box in pixels. Defaults to None.
+            height (int or float, optional): The height of the bounding box in pixels. Defaults to None.
+            polygons (list of dicts, optional): The polygon geometry. Defaults to None.
+            center (list of ints or floats, optional): The center point of the polygon instance. Defaults to None.
             user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
         """
         self.add_2d_keypoints(
             sensor_id=sensor_id,
             label_id=label_id,
             classification=classification,
+            keypoints=keypoints,
+            confidence=confidence,
             top=top,
             left=left,
             width=width,
             height=height,
-            keypoints=keypoints,
-            confidence=confidence,
+            polygons=polygons,
+            center=center,
             user_attrs=user_attrs,
         )
 
     def add_inference_2d_polygon_list(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         polygons: List[Dict[POLYGON_VERTICES_KEYS, List[Tuple[Union[int, float]]]]],
@@ -325,7 +381,7 @@ class InferencesFrame(InferenceLabelSet):
 
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             polygons (list of dicts): The polygon geometry
@@ -346,7 +402,7 @@ class InferencesFrame(InferenceLabelSet):
     def add_inference_2d_semseg(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         mask_url: Optional[str] = None,
         mask_data: Optional[Any] = None,
@@ -355,7 +411,7 @@ class InferencesFrame(InferenceLabelSet):
         """Add an inference for 2D semseg.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             mask_url (str, optional): URL to the pixel mask png.
             mask_data (ndarray, optional): ndarray of pixel mask data, shape [height, width]
@@ -374,7 +430,7 @@ class InferencesFrame(InferenceLabelSet):
     def add_inference_2d_instance_seg(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         mask_url: Optional[str] = None,
         mask_data: Optional[Any] = None,
@@ -384,7 +440,7 @@ class InferencesFrame(InferenceLabelSet):
         """Add an inference for 2D instance segmentation.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             mask_url (str, optional): URL to the pixel mask png.
             mask_data (ndarray, optional): ndarray of pixel mask data, shape [height, width]
@@ -415,7 +471,7 @@ class InferencesFrame(InferenceLabelSet):
     def add_inference_2d_classification(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         confidence: float,
@@ -424,7 +480,7 @@ class InferencesFrame(InferenceLabelSet):
         """Add an inference for 2D classification.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             confidence (float): The confidence between 0.0 and 1.0 of the prediction
@@ -482,6 +538,9 @@ class Inferences:
     _temp_frame_asset_file_names: List[str]
     _temp_frame_embeddings_file_names: List[str]
     backfilled_frame_embs_flag: bool
+    backfilled_crops_flag: bool
+    crop_embedding_dim: Optional[int]
+    frame_embedding_dim: Optional[int]
 
     def __init__(self) -> None:
         if not _is_one_gb_available():
@@ -509,6 +568,9 @@ class Inferences:
         self._temp_frame_embeddings_file_names = []
         self._temp_frame_asset_file_names = []
         self.backfilled_frame_embs_flag = False
+        self.backfilled_crops_flag = False
+        self.frame_embedding_dim = None
+        self.crop_embedding_dim = None
 
     def _cleanup_temp_dir(self) -> None:
         mark_temp_directory_complete(self.temp_file_path)
@@ -698,6 +760,10 @@ class Inferences:
                     f"Inferences must be submitted as the positive class with confidence 0 to 1 for binary classification project {project_info['id']}. Inferences will be interpreted by Aquarium using the decision threshold."
                 )
 
+        if self.backfilled_crops_flag:
+            print(
+                "WARNING: One or more Frames contained Frame Embeddings with Crops with no Crop Embeddings. All crops without Crop Embeddings have been backfilled with a default Embedding Vector"
+            )
         if self.backfilled_frame_embs_flag:
             print(
                 "WARNING: Crop Embeddings were provided but no Frame Embeddings were provided. All inf frames have backfilled with a default Frame Embedding Vector"
@@ -721,12 +787,59 @@ class Inferences:
                 f"Attempted to add duplicate label id(s): {duplicate_label_ids}"
             )
 
+        self.backfilled_crops_flag = frame.validate_and_backfill_crop_embeddings()
         self._frames.append(frame)
         self._frame_ids_set.add(frame.frame_id)
         self._label_ids_set.update(frame._label_ids_set)
         self._label_class_ids_set.update(frame._seen_label_class_ids)
         self._label_classes_set.update(frame._all_label_classes())
         self._frame_summaries.append(frame._to_summary())
+
+        if self.frame_embedding_dim is None:
+            self.frame_embedding_dim = frame.frame_embedding_dim
+        elif self.frame_embedding_dim != frame.frame_embedding_dim:
+            raise Exception(
+                f"Length of frame embeddings must be the same, existing embeddings are of dimension {self.frame_embedding_dim} but new embedding has dimension {frame.frame_embedding_dim}"
+            )
+
+        if len(frame._label_ids_set) > 0:
+            if len(frame.crop_embeddings) > 0 and len(frame.crop_embeddings) != len(
+                frame._label_ids_set
+            ):
+                raise Exception(
+                    f"Embeddings must be provided for all crops or no crops. Embeddings are provided for partial crops for frame {frame.frame_id}."
+                )
+
+            crop_embedding_dim = (
+                frame.crop_embedding_dim if frame.crop_embedding_dim else -1
+            )
+            if self.crop_embedding_dim is None:
+                self.crop_embedding_dim = crop_embedding_dim
+            elif self.crop_embedding_dim != crop_embedding_dim:
+                raise Exception(
+                    f"Length of crop embeddings must be the same, existing embeddings are of dimension {self.crop_embedding_dim} but new embedding has dimension {crop_embedding_dim}"
+                )
+
+        if len(self._frames) >= MAX_FRAMES_PER_BATCH:
+            self._flush_to_disk()
+
+    def delete_frame(self, frame_id: str) -> None:
+        """Delete a frame from this dataset.
+
+        Args:
+            frame_id (str): id of frame to delete.
+        """
+
+        if frame_id in self._frame_ids_set:
+            raise Exception("Attempted to add and delete same frame id.")
+
+        self._frames.append(
+            InferencesFrame(
+                frame_id=frame_id, reuse_latest_embedding=True, update_type="DELETE"
+            )
+        )
+        self._frame_ids_set.add(frame_id)
+
         if len(self._frames) >= MAX_FRAMES_PER_BATCH:
             self._flush_to_disk()
 
