@@ -25,11 +25,13 @@ except ImportError:
 
 
 from .util import (
+    DEFAULT_SENSOR_ID,
     _is_one_gb_available,
     add_object_user_attrs,
     assert_valid_name,
     create_temp_directory,
     mark_temp_directory_complete,
+    extract_illegal_characters,
     POLYGON_VERTICES_KEYS,
     KEYPOINT_KEYS,
     MAX_FRAMES_PER_BATCH,
@@ -64,6 +66,7 @@ class BaseLabelSet:
     crop_embeddings: List[CropEmbeddingDict]
     _seen_label_class_ids: Set[int]
     reuse_latest_embedding: bool
+    crop_embedding_dim: Optional[int]
 
     def __init__(
         self,
@@ -75,8 +78,11 @@ class BaseLabelSet:
         if not isinstance(frame_id, str):
             raise Exception("frame ids must be strings")
 
-        if "/" in frame_id:
-            raise Exception("frame ids cannot contain slashes (/)")
+        illegal_frame_id_characters = extract_illegal_characters(frame_id)
+        if illegal_frame_id_characters:
+            raise Exception(
+                f"frame ids cannot contain the following character(s): {illegal_frame_id_characters}"
+            )
 
         self.frame_id = frame_id
         self.crop_type = crop_type
@@ -88,6 +94,7 @@ class BaseLabelSet:
         self.reuse_latest_embedding = (
             False if reuse_latest_embedding is None else reuse_latest_embedding
         )
+        self.crop_embedding_dim = None
 
     def add_crop_embedding(
         self, *, label_id: str, embedding: List[float], model_id: str = ""
@@ -102,6 +109,13 @@ class BaseLabelSet:
         if not embedding or len(embedding) > 12000:
             raise Exception("Length of embeddings should be between 0 and 12,000")
 
+        if not self.crop_embedding_dim:
+            self.crop_embedding_dim = len(embedding)
+        elif self.crop_embedding_dim != len(embedding):
+            raise Exception(
+                f"Length of embeddings must be the same, existing embeddings are of dimension {self.crop_embedding_dim} but new embedding has dimension {len(embedding)}"
+            )
+
         # TODO: check label id present
         self.crop_embeddings.append(
             {
@@ -111,6 +125,7 @@ class BaseLabelSet:
                 "date_generated": str(datetime.datetime.now()),
             }
         )
+        self.reuse_latest_embedding = False
 
     def _all_label_classes(self) -> List[str]:
         return list(
@@ -119,6 +134,16 @@ class BaseLabelSet:
             )
         )
 
+    def _validate_label_inputs(self, *, label_id: str) -> None:
+        if not isinstance(label_id, str):
+            raise Exception("label ids must be strings")
+
+        illegal_label_id_characters = extract_illegal_characters(label_id)
+        if illegal_label_id_characters:
+            raise Exception(
+                f"frame ids cannot contain the following character(s): {illegal_label_id_characters}"
+            )
+
     def to_dict(self) -> Dict[str, Any]:
         """Convert this label set into a dictionary representation.
 
@@ -160,7 +185,7 @@ class BaseLabelSet:
     def add_2d_line_segment(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         x1: Union[int, float],
@@ -175,7 +200,7 @@ class BaseLabelSet:
         """Add a 2D line segment.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             x1 (int or float): The x-coord of the first vertex in pixels
@@ -187,10 +212,7 @@ class BaseLabelSet:
             user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
         """
 
-        if not isinstance(label_id, str):
-            raise Exception("label ids must be strings")
-        if "/" in label_id:
-            raise Exception("label ids cannot contain slashes (/)")
+        self._validate_label_inputs(label_id=label_id)
 
         if not isinstance(classification, str):
             raise Exception("classifications must be strings")
@@ -235,7 +257,7 @@ class BaseLabelSet:
     def add_2d_bbox(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         top: Union[int, float],
@@ -251,7 +273,7 @@ class BaseLabelSet:
         """Add a 2D bounding box.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             top (int or float): The top of the box in pixels
@@ -264,10 +286,7 @@ class BaseLabelSet:
             user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
         """
 
-        if not isinstance(label_id, str):
-            raise Exception("label ids must be strings")
-        if "/" in label_id:
-            raise Exception("label ids cannot contain slashes (/)")
+        self._validate_label_inputs(label_id=label_id)
 
         if not isinstance(classification, str):
             raise Exception("classifications must be strings")
@@ -324,7 +343,7 @@ class BaseLabelSet:
     def add_text_token(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         index: int,
         token: str,
@@ -336,7 +355,7 @@ class BaseLabelSet:
         """Add a label for a text token.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             index (int): the index of this token in the text
             token (str): the text content of this token
@@ -345,10 +364,7 @@ class BaseLabelSet:
             confidence (float): confidence of prediction
             user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
         """
-        if not isinstance(label_id, str):
-            raise Exception("label ids must be strings")
-        if "/" in label_id:
-            raise Exception("label ids cannot contain slashes (/)")
+        self._validate_label_inputs(label_id=label_id)
 
         if not isinstance(classification, str):
             raise Exception("classifications must be strings")
@@ -412,10 +428,7 @@ class BaseLabelSet:
         if coord_frame_id is None:
             coord_frame_id = "world"
 
-        if not isinstance(label_id, str):
-            raise Exception("label ids must be strings")
-        if "/" in label_id:
-            raise Exception("label ids cannot contain slashes (/)")
+        self._validate_label_inputs(label_id=label_id)
 
         if not isinstance(classification, str):
             raise Exception("classifications must be strings")
@@ -471,15 +484,19 @@ class BaseLabelSet:
     def add_2d_keypoints(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
-        top: Union[int, float],
-        left: Union[int, float],
-        width: Union[int, float],
-        height: Union[int, float],
         keypoints: List[Dict[KEYPOINT_KEYS, Union[int, float, str]]],
         confidence: Optional[float] = None,
+        top: Optional[Union[int, float]] = None,
+        left: Optional[Union[int, float]] = None,
+        width: Optional[Union[int, float]] = None,
+        height: Optional[Union[int, float]] = None,
+        polygons: Optional[
+            List[Dict[POLYGON_VERTICES_KEYS, List[Tuple[Union[int, float]]]]]
+        ] = None,
+        center: Optional[List[Union[int, float]]] = None,
         user_attrs: Optional[Dict[str, Any]] = None,
     ) -> None:
         """Add an inference for a 2D keypoints task.
@@ -490,30 +507,25 @@ class BaseLabelSet:
             'name': string name of the keypoint
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
-            top (int or float): The top of the box in pixels
-            left (int or float): The left of the box in pixels
-            width (int or float): The width of the box in pixels
-            height (int or float): The height of the box in pixels
             keypoints (list of dicts): The keypoints of this detection
             confidence (float): The confidence between 0.0 and 1.0 of the prediction
+            top (int or float, optional): The top of the bounding box in pixels. Defaults to None.
+            left (int or float, optional): The left of the bounding box in pixels. Defaults to None.
+            width (int or float, optional): The width of the bounding box in pixels. Defaults to None.
+            height (int or float, optional): The height of the bounding box in pixels. Defaults to None.
+            polygons (list of dicts, optional): The polygon geometry. Defaults to None.
+            center (list of ints or floats, optional): The center point of the polygon instance. Defaults to None.
             user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
         """
 
-        if not isinstance(label_id, str):
-            raise Exception("label ids must be strings")
-        if "/" in label_id:
-            raise Exception("label ids cannot contain slashes (/)")
+        self._validate_label_inputs(label_id=label_id)
 
         if not isinstance(classification, str):
             raise Exception("classifications must be strings")
 
-        for dim in [top, left, width, height]:
-            if not is_valid_number(dim):
-                raise Exception("all bounding box dimensions must be valid int/float")
-
         keypoint_names_set = set()
         for keypoint in keypoints:
             x = keypoint["x"]
@@ -529,14 +541,44 @@ class BaseLabelSet:
                 raise Exception("keypoint names must be unique within a label")
             keypoint_names_set.add(name)
 
-        attrs = {
-            "top": top,
-            "left": left,
-            "width": width,
-            "height": height,
+        attrs: Dict[str, Any] = {
             "keypoints": keypoints,
         }
 
+        if (
+            top is not None
+            and left is not None
+            and width is not None
+            and height is not None
+        ):
+            for dim in [top, left, width, height]:
+                if not is_valid_number(dim):
+                    raise Exception(
+                        "all bounding box dimensions must be valid int/float"
+                    )
+            attrs["top"] = top
+            attrs["left"] = left
+            attrs["width"] = width
+            attrs["height"] = height
+
+        if polygons:
+            for polygon in polygons:
+                for vertex in polygon["vertices"]:
+                    for num in vertex:
+                        if not is_valid_number(num):
+                            raise Exception(
+                                "all polygon vertex coordinates must be valid int/float"
+                            )
+            attrs["polygons"] = polygons
+
+        if center is not None:
+            for num in center:
+                if not is_valid_number(num):
+                    raise Exception(
+                        "polygon center coordinates must be valid int/float"
+                    )
+            attrs["center"] = center
+
         if confidence is not None:
             if not is_valid_float(confidence):
                 raise Exception("confidence must be floats")
@@ -562,7 +604,7 @@ class BaseLabelSet:
     def add_2d_polygon_list(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         polygons: List[Dict[POLYGON_VERTICES_KEYS, List[Tuple[Union[int, float]]]]],
@@ -590,7 +632,7 @@ class BaseLabelSet:
 
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             polygons (list of dicts): The polygon geometry
@@ -599,10 +641,7 @@ class BaseLabelSet:
             user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
         """
 
-        if not isinstance(label_id, str):
-            raise Exception("label ids must be strings")
-        if "/" in label_id:
-            raise Exception("label ids cannot contain slashes (/)")
+        self._validate_label_inputs(label_id=label_id)
 
         if not isinstance(classification, str):
             raise Exception("classifications must be strings")
@@ -650,7 +689,7 @@ class BaseLabelSet:
     def add_2d_semseg(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         mask_url: Optional[str] = None,
         mask_data: Optional[Any] = None,
@@ -660,7 +699,7 @@ class BaseLabelSet:
         Add an inference for 2D semseg. These should provide either a mask_url or a mask_data.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             mask_url (str, optional): URL to the pixel mask png.
             mask_data (ndarray, optional): ndarray of pixel mask data, shape [height, width]
@@ -669,10 +708,7 @@ class BaseLabelSet:
                 None: no change
         """
 
-        if not isinstance(label_id, str):
-            raise Exception("label ids must be strings")
-        if "/" in label_id:
-            raise Exception("label ids cannot contain slashes (/)")
+        self._validate_label_inputs(label_id=label_id)
 
         if mask_data is None and mask_url is None:
             raise Exception("Please provide either a mask_data or a mask_url")
@@ -746,7 +782,7 @@ class BaseLabelSet:
     def add_2d_instance_seg(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         mask_url: Optional[str] = None,
         mask_data: Optional[Any] = None,
@@ -757,7 +793,7 @@ class BaseLabelSet:
         Add an label for 2D instance segmentation. These should provide either a mask_url or a mask_data, and a instance mapping.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             mask_url (str, optional): URL to the pixel mask png.
             mask_data (ndarray, optional): ndarray of pixel mask data, shape [height, width]
@@ -777,10 +813,7 @@ class BaseLabelSet:
                 None: no change
         """
 
-        if not isinstance(label_id, str):
-            raise Exception("label ids must be strings")
-        if "/" in label_id:
-            raise Exception("label ids cannot contain slashes (/)")
+        self._validate_label_inputs(label_id=label_id)
 
         if mask_data is None and mask_url is None:
             raise Exception("Please provide either a mask_data or a mask_url")
@@ -874,7 +907,7 @@ class BaseLabelSet:
     def add_2d_classification(
         self,
         *,
-        sensor_id: str,
+        sensor_id: str = DEFAULT_SENSOR_ID,
         label_id: str,
         classification: str,
         confidence: Optional[float] = None,
@@ -884,17 +917,14 @@ class BaseLabelSet:
         """Add an inference for 2D classification.
 
         Args:
-            sensor_id (str): sensor_id
+            sensor_id (str): ID of sensor that generates this datapoint. Must match base frame sensor_id. Do not use arg if not used in base frame.
             label_id (str): label_id which is unique across datasets and inferences.
             classification (str): the classification string
             confidence (float): The confidence between 0.0 and 1.0 of the prediction
             user_attrs (dict, optional): Any additional label-level metadata fields. Defaults to None.
         """
 
-        if not isinstance(label_id, str):
-            raise Exception("label ids must be strings")
-        if "/" in label_id:
-            raise Exception("label ids cannot contain slashes (/)")
+        self._validate_label_inputs(label_id=label_id)
 
         if not isinstance(classification, str):
             raise Exception("classifications must be strings")
@@ -949,10 +979,7 @@ class BaseLabelSet:
         if coord_frame_id is None:
             coord_frame_id = "world"
 
-        if not isinstance(label_id, str):
-            raise Exception("label ids must be strings")
-        if "/" in label_id:
-            raise Exception("label ids cannot contain slashes (/)")
+        self._validate_label_inputs(label_id=label_id)
 
         if not isinstance(classification, str):
             raise Exception("classifications must be strings")
