@@ -358,9 +358,7 @@ class MetricsManager:
         )
 
         is_streaming = base_dataset_info.get("is_streaming", False)
-        latest_window = (
-            inferences_info.get("latest_fit_version", "0") if is_streaming else "0"
-        )
+        latest_window = str((inferences_info.get("ingest_agg_windows") or [0])[-1])
         bbox_metrics_metadata = inferences_info.get("bbox_metrics_metadata")
         if bbox_metrics_metadata is None:
             raise Exception("No valid metrics computed.")
