@@ -16,11 +16,12 @@ import datetime
 from google.resumable_media.requests import ResumableUpload
 from google.resumable_media.common import InvalidResponse, DataCorruption
 
-retry_strategy = Retry(
+retry_strategy = Retry(  # type: ignore
     total=4,
     backoff_factor=1,
     status_forcelist=[404, 429, 500, 502, 503, 504],
     method_whitelist=["HEAD", "GET", "PUT", "POST", "DELETE", "OPTIONS", "TRACE"],
+    raise_on_status=False,
 )
 retry_adapter = HTTPAdapter(max_retries=retry_strategy)
 requests_retry = requests.Session()
@@ -33,6 +34,7 @@ MAX_CHUNK_SIZE = int(pow(2, 23))  # 8 MiB
 DEFAULT_SAMPLING_THRESHOLD = 0.5
 
 DUMMY_FRAME_EMBEDDING = [0.0, 0.0, 0.0]
+DEFAULT_SENSOR_ID = "DEFAULT"
 
 
 def _upload_local_files(
@@ -323,6 +325,16 @@ def maybe_parse_json_vals(dict: Dict[str, Any]) -> Dict[str, Any]:
     return new_dict
 
 
+def extract_illegal_characters(el_id: str) -> List[str]:
+    # prohibit characters that violate https://cloud.google.com/storage/docs/naming-objects#objectnames
+    # and commas and slashes as they are used for internal aquarium aggregations
+    # also ban all non-ascii characters
+    matches = re.findall(
+        "[^\x00-\xFF]|#|[[]|[]]|[?]|[*]|,|/|[\x7F-\x84]|[\x86-\x9F]", el_id
+    )
+    return list(set(matches))
+
+
 # TODO: Is there a way to just infer these from the constant initialized dictionary literal?
 USER_METADATA_PRIMITIVE = Union[str, int, float, bool]
 USER_METADATA_PRIMITIVE_TYPES = Literal["str", "int", "float", "bool"]
